package org.w3c.rdfpic.core;
import java.net.URL;
/**
 * Image loader is a class that places load requests for new images in a new thread so that the UI does not lock up
 * when a file takes long to load.  This also means that the user interface must take mesures to insure that no changes
 * are applied while loading.  (Though they should not cause any crashes in RdfPicCore).
 */

public class ImageLoader extends Thread
{
    RdfPicCore core;
    boolean syncNeeded;
    boolean quitNow;
    /**
     * Creates a new ImageLoader.
     * @param core The core to interact with.
     */
    public ImageLoader(RdfPicCore core)
    {
	super("LoaderThread");
	this.core=core;
    }
    /**
     * Starts the loading thread, in which load requests will later execute.
     */
    public void run()
    {
	syncLoop();
    }

    /**
     * Waits until something happens, process it, and start over...
     */
    private synchronized void syncLoop()
    {
	quitNow=false;
	do
	    {
		sync();
		if(!interrupted())
		    try {wait();}catch(InterruptedException e){}
	    } while(!quitNow);
    }

    /**
     * Synchronizes image - finish loading.
     */
    private void sync()
    {
	while (syncNeeded)
	    {
		yield();
		syncNeeded=false;
		core.finishJump();
	    }
    }
    /**
     * Shuts down load thread. (IDEA deamon thread easier - maybe io problems though?)
     */
    void quit()//call from other thread
    {
	quitNow=true;
	try {
	    interrupt();
	} catch (SecurityException err){
	    err.printStackTrace();
	    System.err.print("EXCEPTION CAUGHT!!");
	}
    }
    /**
     * Puts a load request.  The active image will be (re)loaded.
     */

    void reload()//call from other thread
    {
	syncNeeded=true;
	try {
	    interrupt();
	} catch (SecurityException err){
	    err.printStackTrace();
	    System.err.print("EXCEPTION CAUGHT!!");
	}
    }
}
