package org.w3c.rdfpic.ui;
import org.w3c.rdfpic.core.*;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.event.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.IOException;
import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.TreeSet;


/**
 * A simple UI for RdfPic.
 * It creates a menubar with a '?' (about) 'rdf' and 'image' menus and lets the user use
 * SchemaTabbedPane to edit the rdf in the image.  The SchemaTabbedPane is placed next to a
 * preview of the image in a JSplitPane.  'next' 'prev' and 'apply' command are also available
 * in a toolbar initially under the menubar, as well as a language selection box.
 */


public class SimpleUI extends JFrame implements ActionListener, WindowListener, ItemListener,ImgListener,SchemaListener,Checker
{
    static String titleStr = "Rdfpic 2.1";
    JFileChooser fc;
    ExtensionFileFilter effJpeg,effSchema;
    JButton applyButton,nextButton,prevButton;
    JComboBox langsel;
    ScaledPhoto photo;//in the left top, the preview picture
    SchemaTabbedPane dataPane;//the right section, the data input section.
    JMenuBar menuBar;
    JMenu fileMenu, imageMenu,aboutMenu,/*saveSchemaMenu,*/ toggleSchemaMenu,selectImageMenu;
    JMenuItem applyItem, nextItem, prevItem;
    JMenuItem extractRaw,insertRaw,importRdf,exportRdf;
    JCheckBoxMenuItem[] toggleSchemaItem;
    //    JMenuItem[] saveSchemaItems;
    ButtonGroup selectImageGroup;
    JRadioButtonMenuItem[] selectImageItems;
    RdfPicCore core;
    boolean ignoreToggle;

    static final String separatorStr = " - ";

    public SimpleUI()
    {
	super();
    }

    //functions for WindowListener..
    public void windowActivated(WindowEvent e){}
    public void windowDeactivated(WindowEvent e){}
    public void windowIconified(WindowEvent e){}
    public void windowDeiconified(WindowEvent e){}
    public void windowOpened(WindowEvent e){}
    public void windowClosed(WindowEvent e){}
    public void windowClosing(WindowEvent e)
    {
	quit();
    }

    /**
     * Activates the UI
     */
    public void go(RdfPicCore core)
    {
	fc=new JFileChooser();
	fc.setMultiSelectionEnabled(true);
        effJpeg=new ExtensionFileFilter();
	effJpeg.addExtension("jpg");
	effJpeg.addExtension("jpeg");
	effJpeg.setDescription("Jpeg images");
        effSchema=new ExtensionFileFilter();
	effSchema.addExtension("schema");
	effSchema.addExtension("sch");
	effSchema.setDescription("Schema files");

	ignoreToggle=false;
	this.core=core;
	core.addImgListener(this);
	core.addSchemaListener(this);
	//in order to catch a close-window command
        addWindowListener(this);
	
	JPanel container; //the ueber-container. :-)
	JSplitPane mainPane;//the program's main (img/rdf) container
	dataPane = new SchemaTabbedPane(core);//contains the tabbed pane
	//where you fill in stuff
	dataPane.addBuiltin("subject",new DCSubjectCboxMaker());
	
	//set up the image picture
	photo = new ScaledPhoto();
	try {
	    java.awt.Image image = core.getImg();
	    if (image==null) {
		photo.setAlt("No Image loaded.");
		photo.setIcon(null);
	    } else
		photo.setIcon(new ScalableImageIcon(image));
	} catch (RdfPicError rpe) {
	    photo.setAlt("Error while loading Image");
	}
	
	//the main window is a split pane between the rdf data entry and
	//the image preview
	mainPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,dataPane,photo);
	
	//A toolbar to goto the next/previous image or to apply rdf-changes
	JToolBar toolBar = new JToolBar();
	
	//now for some buttons for our toolbar
	prevButton= new JButton("Previous");
	prevButton.setMnemonic(KeyEvent.VK_P);
	prevButton.setActionCommand("previous");
	prevButton.setToolTipText("Go to the previous picture");
	prevButton.addActionListener(this);
	
	applyButton= new JButton("Apply");
	applyButton.setMnemonic(KeyEvent.VK_A);
	applyButton.setActionCommand("apply");
	applyButton.setToolTipText("Apply changes made to Rdf-data");
	applyButton.addActionListener(this);
	
	nextButton= new JButton("Next");
	nextButton.setMnemonic(KeyEvent.VK_N);
	nextButton.setActionCommand("next");
	nextButton.setToolTipText("Go to the next picture");
	nextButton.addActionListener(this);

	langsel=new MemComboBox();
	langsel.addItem("en");
	langsel.addItem("fr");
	langsel.addItem("de");
	langsel.addItem("nl");
	langsel.setSelectedIndex(0);
	langsel.setEditable(true);
	langsel.setMaximumSize(new Dimension(60,32));

	//the buttons incidentally are part of the toolbar
	toolBar.add(applyButton);
	toolBar.add(prevButton);
	toolBar.add(nextButton);
	toolBar.add(langsel);
	
	//to be able to drag the toolbar, we define a container and
	//put the main Pane in the center and the toolbar to the north
	//now when we drag the toolbar, it can be placed on any of the four
	//sides of the main pane.
	container= new JPanel(new BorderLayout());
	container.add(mainPane, BorderLayout.CENTER);
	container.add(toolBar, BorderLayout.NORTH);
	
	//We also want a few menu's
	menuBar=new JMenuBar();
	JMenuItem menuItem;
	
	//An Image menu...
	imageMenu=new JMenu("Image");
	imageMenu.setMnemonic(KeyEvent.VK_I);
	
	//...with several menu items.
	menuItem=new JMenuItem("Previous",KeyEvent.VK_P);
	menuItem.setActionCommand("previous");
	menuItem.addActionListener(this);
	imageMenu.add(prevItem=menuItem);
	
	menuItem=new JMenuItem("Next",KeyEvent.VK_N);
	menuItem.setActionCommand("next");
	menuItem.addActionListener(this);
	imageMenu.add(nextItem=menuItem);
	
	menuItem=new JMenuItem("Apply",KeyEvent.VK_A);
	menuItem.setActionCommand("apply");
	menuItem.addActionListener(this);
	imageMenu.add(applyItem=menuItem);

	selectImageMenu=new JMenu("Jump to");
	selectImageMenu.setMnemonic(KeyEvent.VK_J);
	selectImageItems = new JRadioButtonMenuItem[core.numFiles()];
	genSelectImageItems();
	imageMenu.add(selectImageMenu);
	
	imageMenu.addSeparator();

	menuItem = new JMenuItem("Extract Raw Data",KeyEvent.VK_E);
	menuItem.setActionCommand("extractRaw");
	menuItem.addActionListener(this);
	imageMenu.add(menuItem);
	extractRaw=menuItem;

	menuItem = new JMenuItem("Insert Raw Data",KeyEvent.VK_I);
	menuItem.setActionCommand("insertRaw");
	menuItem.addActionListener(this);
	imageMenu.add(menuItem);
	insertRaw=menuItem;

	imageMenu.addSeparator();

	menuItem = new JMenuItem("Load New Image",KeyEvent.VK_L);
	menuItem.setActionCommand("loadImg");
	menuItem.addActionListener(this);
	imageMenu.add(menuItem);

	//A File Menu...
	fileMenu=new JMenu("Rdf");
	fileMenu.setMnemonic(KeyEvent.VK_R);
	
	menuItem=new JMenuItem("Load Schema",KeyEvent.VK_L);
	menuItem.setActionCommand("loadSchema");
	menuItem.addActionListener(this);
	fileMenu.add(menuItem);
	
	toggleSchemaMenu = new JMenu("Toggle Schema");
	toggleSchemaMenu.setMnemonic(KeyEvent.VK_T);
	genToggleSchema();
	fileMenu.add(toggleSchemaMenu);

	fileMenu.addSeparator();
	
	menuItem= new JMenuItem("Import RDF",KeyEvent.VK_I);
	menuItem.setActionCommand("importRDF");
	menuItem.addActionListener(this);
	fileMenu.add(menuItem);
	importRdf=menuItem;
	
	menuItem= new JMenuItem("Export RDF",KeyEvent.VK_E);
	menuItem.setActionCommand("exportRDF");
	menuItem.addActionListener(this);
	fileMenu.add(menuItem);
	exportRdf=menuItem;
	
	fileMenu.addSeparator();
	
	menuItem=new JMenuItem("Quit",KeyEvent.VK_Q);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,ActionEvent.ALT_MASK));
	menuItem.setActionCommand("quit");
	menuItem.addActionListener(this);
	fileMenu.add(menuItem);
	
	
	//An "About" menu
	aboutMenu=new JMenu("?");
	
	menuItem=new JMenuItem("About "+titleStr);
	menuItem.setActionCommand("about");
	menuItem.addActionListener(this);
	aboutMenu.add(menuItem);
	
	menuBar.add(fileMenu);
	menuBar.add(imageMenu);
        menuBar.add(Box.createHorizontalGlue());
	menuBar.add(aboutMenu);
	
	
	//Code for initially enabling/disabling buttons
	checkEnabledStatus();
	writeEnable(core.canWrite());
	
	getContentPane().add(container);
	setJMenuBar(menuBar);
	
	
	setTitle(titleStr + separatorStr + core.fileName(core.getPos()));
	pack();
	setVisible(true);
    }
    void checkEnabledStatus()
    {
	checkEnabledStatus(core.getPos());
    }

    void checkEnabledStatus(int index)//check whether previous/next/apply should be enabled
    {
	int numFiles=core.numFiles();
	if(index<0)
	    {
		prevItem.setEnabled(false);
		prevButton.setEnabled(false);
	    }
	else
	    {
		prevItem.setEnabled(true);
		prevButton.setEnabled(true);
	    }
	if(index>=numFiles-1)
	    {
		nextItem.setEnabled(false);
		nextButton.setEnabled(false);
	    }
	else
	    {
		nextItem.setEnabled(true);
		nextButton.setEnabled(true);
	    }
	writeEnable(core.canWrite());
    }
    
    void genToggleSchema()
    {
	int size = core.numSchemas();
	JCheckBoxMenuItem cbMenuItem;
	toggleSchemaMenu.removeAll();
	toggleSchemaItem= new JCheckBoxMenuItem[size];
	for(int i=0;i<size;++i)
	    {
		cbMenuItem = new JCheckBoxMenuItem(core.schemaName(i));
		cbMenuItem.setSelected(core.isEnabled(i));
		cbMenuItem.addItemListener(this);
		toggleSchemaMenu.add(cbMenuItem);
		toggleSchemaItem[i]=cbMenuItem;
	    }
    }

    void genSelectImageItems()
    {
	JRadioButtonMenuItem rbMenuItem;
	int numF=core.numFiles();
	selectImageItems = new JRadioButtonMenuItem[numF+1];
	selectImageGroup = new ButtonGroup();
	selectImageMenu.removeAll();

	rbMenuItem = new JRadioButtonMenuItem("<none>");
	selectImageMenu.add(rbMenuItem);
	selectImageItems[0]=rbMenuItem;
	selectImageGroup.add(rbMenuItem);
	rbMenuItem.addActionListener(this);
	rbMenuItem.setActionCommand("jumpTo");
	for(int i=0;i<numF;++i)
	    {
		rbMenuItem = new JRadioButtonMenuItem(core.fileName(i));
		rbMenuItem.addActionListener(this);
		rbMenuItem.setActionCommand("jumpTo");
		selectImageMenu.add(rbMenuItem);
		selectImageGroup.add(rbMenuItem);
		selectImageItems[i+1]=rbMenuItem;
	    }
	selectImageItems[core.getPos()+1].setSelected(true);
    } 

    public void addingSchema()
    {
	genToggleSchema();
    }

    public void enablingSchema(int index)
    {
	toggleSchemaItem[index].setSelected(true);
    }

    public void disablingSchema(int index)
    {
	toggleSchemaItem[index].setSelected(false);
    }

    public void itemStateChanged(ItemEvent e)
    {
	JMenuItem source = (JMenuItem)(e.getSource());
	int index=-1;
	for(int i=0;i<toggleSchemaItem.length;i++)
	    if (toggleSchemaItem[i]==source) {index=i;break;}
	if(e.getStateChange()==ItemEvent.SELECTED)
	    core.enableSchema(index);
	else
	    core.disableSchema(index);
    } 
    
    void jumpSafe(boolean safe)
    {
	applyButton.setEnabled(safe);
	applyItem.setEnabled(safe);
	extractRaw.setEnabled(safe);
	insertRaw.setEnabled(safe);
	importRdf.setEnabled(safe);
	exportRdf.setEnabled(safe);
    }

    void writeEnable(boolean safe)
    {
	applyButton.setEnabled(safe);
	applyItem.setEnabled(safe);
	extractRaw.setEnabled(safe);
	insertRaw.setEnabled(safe);
    }


    public void isJumping(int newindex)
    {
	checkEnabledStatus(newindex);
	jumpSafe(false);
	if(!selectImageItems[newindex+1].isSelected())
	    selectImageItems[newindex+1].setSelected(true);
	setTitle(titleStr + separatorStr + core.fileName(newindex));
	photo.setAlt("Loading...");
	photo.setIcon(null);
    }
    public void imgVUpdate()
    {
	genSelectImageItems();
	checkEnabledStatus();
    }

    public void hasJumped(String errMsg)
    {  
	if(errMsg!=null)
	    {
		JOptionPane.showMessageDialog(this,
					      "There was an error loading this file.\n"+errMsg,
					      "Error Loading "+core.fileName(core.getPos()),
					      JOptionPane.ERROR_MESSAGE);
		photo.setIcon(null);
		photo.setAlt("Unable to load "+core.fileName(core.getPos()));
		return;
	    }
	try {
	    langsel.setSelectedItem(core.getLang());
	    java.awt.Image img = core.getImg();
	    if (img!=null) photo.setIcon(new ScalableImageIcon(img));
	    else {
		photo.setAlt("No Image loaded.");
		photo.setIcon(null);
	    }
	} catch (Exception exp) {
	    JOptionPane.showMessageDialog(this,
					  "There was an unexpected error loading this file.\n",
					  "Error Loading "+core.fileName(core.getPos()),
					  JOptionPane.ERROR_MESSAGE);
	    photo.setIcon(null);
	    photo.setAlt("Error while loading");
	    return;
	}
	jumpSafe(true);
	writeEnable(core.canWrite());
    }

    public void actionPerformed(ActionEvent e)
    {
	String act = e.getActionCommand();

	if (act.equals("previous"))  core.jumpRel(-1);
	else if (act.equals("next")) core.jumpRel(1);
	else if (act.equals("jumpTo"))
	    {
		JRadioButtonMenuItem source = (JRadioButtonMenuItem)e.getSource();
		int newindex=-1;
		for(int i=0;i<selectImageItems.length;++i)
		    if(selectImageItems[i]==source) {newindex=i-1;break;}
		core.jumpTo(newindex);
	    }
	else if (act.equals("apply"))
	    {
		core.setLang((String)langsel.getSelectedItem());
		try {core.saveData();}
		catch (IOException ioe) {
			JOptionPane.showMessageDialog(this,
						      "There was a write error inserting the meta-data:\n" +
						      "\""+ioe.getMessage()+"\"",
						      "Unable to write to "+core.fileName(core.getPos()),
						      JOptionPane.ERROR_MESSAGE);
		}
		catch (RdfPicError rpe) 
		    {
			JOptionPane.showMessageDialog(this,
						      "There was an error inserting the meta-data:\n" +
						      "\""+rpe.getMessage()+"\"",
						      "Error Saving "+core.fileName(core.getPos()),
						      JOptionPane.ERROR_MESSAGE);
		    }
	    }
	else if (act.equals("quit")) quit();
	else if (act.equals("about")) AboutFrame.show(this);
	else if (act.equals("loadSchema"))
	    {
		int choice = JOptionPane.showConfirmDialog(this,"Do you wish to browse the filesystem for a schema definition?",
					 "Load a new Schema",JOptionPane.YES_NO_CANCEL_OPTION);
		if(choice == JOptionPane.NO_OPTION)
		    new CheckedDialog(this,this,act,
			       "Load a New Schema","Enter the URL of the schema you wish to load : ",
			       "Illegal URL","The URL you entered either does not exist, or is unreadable.",
			       new JButton("Load"),true);
		else if(choice == JOptionPane.YES_OPTION) {
		    fc.resetChoosableFileFilters();
		    fc.addChoosableFileFilter(effSchema);
		    if (JFileChooser.APPROVE_OPTION == fc.showDialog(this,"Load")) {
			/* multiple files not supported yet!!!!  FIXME when possible.
			 * File[] files = fc.getSelectedFiles();
			 */
			File[] files=new File[1];
			files[0]=fc.getSelectedFile();
			for (int fileIndex=0;fileIndex<files.length;fileIndex++) {
			    try {
				core.addSchema(new URL("file","",files[fileIndex].toString()));
			    }
			    catch (MalformedURLException mue) {
				JOptionPane.showMessageDialog(this,"Internal Error: "+mue.toString(),
							      "Internal Error",JOptionPane.ERROR_MESSAGE);
			    }
			    catch (IOException err) {
				JOptionPane.showMessageDialog(this,"Unable to read/parse file ("+files[fileIndex]+")","File access error",JOptionPane.WARNING_MESSAGE);
			    }
			}
		    }
		}
		    
	    }
	else if (act.equals("loadImg"))
	    {
		int choice = JOptionPane.showConfirmDialog(this,"Do you wish to browse the filesystem for an Image file?",
					 "Load a new Schema",JOptionPane.YES_NO_CANCEL_OPTION);
		if(choice == JOptionPane.NO_OPTION)
		    new CheckedDialog(this,this,act,
			       "Load a New Image","Enter the URL of the image you wish to load : ",
			       "Illegal URL","The URL you entered either does not exist, or is unreadable.",
			       new JButton("Load"),true);
		else if(choice == JOptionPane.YES_OPTION) {
		    fc.resetChoosableFileFilters();
		    fc.addChoosableFileFilter(effJpeg);
		    if (JFileChooser.APPROVE_OPTION == fc.showDialog(this,"Load")) {
			/* multiple files not supported yet!!!!  FIXME when possible.
			 * File[] files = fc.getSelectedFiles();
			 */
//			File[] files=new File[1];
//			files[0]=fc.getSelectedFile();
		        File[] files = fc.getSelectedFiles();
			int current = core.numFiles();
			for (int fileIndex=0;fileIndex<files.length;fileIndex++) {
			    try {
				core.addURL(new URL("file","",files[fileIndex].toString()));
//				if(fileIndex==files.length-1) {
//				    core.jumpTo(core.numFiles()-1);
//				}
			    }
			    catch (MalformedURLException mue) {
				JOptionPane.showMessageDialog(this,"Internal Error: "+mue.toString(),
							      "Internal Error",JOptionPane.ERROR_MESSAGE);
			    }
			    catch (IOException err) {
				JOptionPane.showMessageDialog(this,"Unable to read/parse file ("+files[fileIndex]+")","File access error",JOptionPane.WARNING_MESSAGE);
			    }
			}
			core.jumpTo(current);
		    }
		}
	    }
	else if (act.equals("exportRDF")) {
	    core.setLang((String)langsel.getSelectedItem());
	    fc.resetChoosableFileFilters();
	    while (JFileChooser.APPROVE_OPTION == fc.showDialog(this,"Export"))
		{
		    try {core.exportRDF(fc.getSelectedFile());}
		    catch (IOException err) {
			JOptionPane.showMessageDialog(this,"Unable to write to file","File access error",JOptionPane.WARNING_MESSAGE);
			continue;
		    }
		    break;
		}
	}
	else if (act.equals("importRDF")) {
	    fc.resetChoosableFileFilters();
	    while (JFileChooser.APPROVE_OPTION == fc.showDialog(this,"Import"))
		{
		    try {core.importRDF(fc.getSelectedFile());}
		    catch (IOException err) {
			JOptionPane.showMessageDialog(this,"Unable to read from file","File access error",JOptionPane.WARNING_MESSAGE);
			continue;
		    } catch (RdfPicError rpe) {
			JOptionPane.showMessageDialog(this,"Unable to parse Rdf from file","Syntax error",JOptionPane.ERROR_MESSAGE);
			break;
		    }
		    langsel.setSelectedItem(core.getLang());
		    break;
		}
	}
	else if (act.equals("extractRaw")) {
	    fc.resetChoosableFileFilters();
	    while (JFileChooser.APPROVE_OPTION == fc.showDialog(this,"Extract to"))
		{
		    try {core.extractRaw(fc.getSelectedFile());}
		    catch (IOException err) {
			JOptionPane.showMessageDialog(this,"Unable to write to file","File access error",JOptionPane.WARNING_MESSAGE);
			continue;
		    } catch (RdfPicError rpe) {
			JOptionPane.showMessageDialog(this,"Unable to read meta-data from Image","File access error",JOptionPane.ERROR_MESSAGE);
			break;
		    }
		    break;
		}
	}
	else if (act.equals("insertRaw")) {
	    fc.resetChoosableFileFilters();
	    while (JFileChooser.APPROVE_OPTION == fc.showDialog(this,"Insert from"))
		{
		    try {core.insertRaw(fc.getSelectedFile());} 
		    catch (PHException phe) {
			JOptionPane.showMessageDialog(this,"Unable to write meta-data to Image","File access error",JOptionPane.WARNING_MESSAGE);
			break;
		    }
		    catch (RdfPicError rpe) {
			JOptionPane.showMessageDialog(this,"Unable to write to Image","File access error",JOptionPane.WARNING_MESSAGE);
			break;
		    }
		    catch (IOException err) {
			JOptionPane.showMessageDialog(this,"Unable to read from file","File access error",JOptionPane.WARNING_MESSAGE);
			continue;
		    }
		    break;
		}
	}
	else yetTodo("Unknown action: "+act);
    }

    public boolean checkInput(String msgID, String input)
    {
	if(msgID.equals("loadSchema"))
	    {
		URL url;
		try {url=new URL(input);}
		catch (MalformedURLException e) {
		    try {url=new URL("file","",input);}
		    catch (MalformedURLException e2) {return false;}
		}
		try {core.addSchema(url);}
		catch (IOException ioe) {return false;}
		return true;
	    }
	else if(msgID.equals("loadImg"))
	    {
		URL url;
		try {url=new URL(input);}
		catch (MalformedURLException e) {
		    try {url=new URL("file","",input);}
		    catch (MalformedURLException e2) {return false;}
		}
		core.addURL(url);
		core.jumpTo(core.numFiles()-1);
		return true;
	    }
	else yetTodo("Unknown dialog: "+msgID);
	return false;
    }

    public void quit()
    {
	dispose();
	System.exit(0);
    }

    private void yetTodo(String msg)
    {
	if (msg==null) msg="";
	JOptionPane.showMessageDialog(this,"This program ain't quite done yet:\n"+msg,"Oops...",JOptionPane.ERROR_MESSAGE);
    }
}

class ExtensionFileFilter extends FileFilter {
    TreeSet extSet;
    String desc;
    public void setDescription(String newdesc) {desc=newdesc;}
    public String getDescription() {return desc;}
    public ExtensionFileFilter() {
	extSet=new TreeSet();
	desc="Extension based file filtering.";
    }
    public void addExtension(String ext) {
	extSet.add(ext.toLowerCase());
    }
    public boolean accept(File f) {
	if (f.isDirectory()) {return true;}
	String fileName = f.getName();
	int dotI = fileName.lastIndexOf(".");
	if (dotI > 0) {
	    return extSet.contains(fileName.substring(dotI+1).toLowerCase());
	} else return false;
    }
}
